
/*
EmonHUB ridi komunikaci po RS485 a posila pozadavky na data jednotlivym nodum
po RS posila data do Raspberry Pi, zobrazeni vybranych hodnot na LCD
automaticka zmena obrazovek, zmena zobrazeni tlacitkem
verze 4
6.9.2015
Tomas Nevrela; tnweb.tode.cz
*/

#include <SoftEasyTransfer.h>
#include <SoftwareSerial.h>
#include <Wire.h>
#include <LiquidCrystal_I2C.h>
SoftwareSerial mySerial(2, 4);//Arduino RX - RS485 RO, Arduino TX - RS485 DI
#define TXenableRS485 3 //RE + DE
#define period 3000 //cas mezi zasilanim pozadavku nodum
#define timeScreen 8000 //cas mezi zmenami obrazovky
#define timeZobraz 800 //cas obnoveni obrazovek
#define maxScreen 7
#define LEDpin 13
#define Tlac A0 //ovladaci tlacitko
#define myNodeID 15   //node ID  
unsigned long currentMillis, previousMillis, lastScreen;
unsigned long rucTime;
unsigned long lastZobraz;
int x = 1;
int screen = 1;
int rucTimeExit = 20000; //cas prepnuti z rucniho do aut. rezimu
boolean rucne = false;
byte stupen[8] = {0b01100, 0b10010, 0b10010, 0b01100, 0b00000, 0b00000, 0b00000, 0b00000};

// struktura pro komunikaci po RS485
struct PayloadTX {
  byte nodeID, address, command, func;
  int data1, data2, data3, data4, data5, data6, data7, data8;
  long data9, data10, data11, data12;
};
PayloadTX emontx;

SoftEasyTransfer ET;
// Nastav pro LCD piny PCF8574T a I2C addresu 0x27(PCF8574AT 0x3f)
//                    addr, en,rw,rs,d4,d5,d6,d7,bl,blpol
LiquidCrystal_I2C lcd(0x3F, 2, 1, 0, 4, 5, 6, 7, 3, POSITIVE);

unsigned char i;
//odesilana data do raspberry pi
union EmonData  //EmonTX
{
  unsigned char item[];
  struct
  {
    int power1;
    int power2;
    int power3;
    int powerAll;
    int Vrms;
    int tarif;
  } val;
} data;

union EmonData1  // Kotel
{
  unsigned char item1[];
  struct
  {
    int TempKotel;
    int TempOut;
    int TempBojler;
    boolean PodStav;
    boolean VentStav;
    boolean TermStav;
    boolean CerpUTStav;
    boolean CerpTUVStav;
    unsigned long PodM;
    unsigned long VentM;
  } val1;
} data1;

union EmonData2  //Solar Inverter
{
  unsigned char item2[];
  struct
  {
    int napeti;
    int proud;
    int vykon;
    unsigned long vyroba;
  } val2;
} data2;


//=========================Setup================================================
void setup() {

  pinMode(TXenableRS485, OUTPUT);
  Serial.begin(9600);
  mySerial.begin(9600); //ser. port pro RS485
  ET.begin(details(emontx), &mySerial);
  digitalWrite(TXenableRS485, LOW);
  pinMode(LEDpin, OUTPUT);
  digitalWrite(LEDpin, HIGH);
  lcd.begin(16, 2); // pocet znaku, pocet radku
  lcd.backlight(); // zapni podsvetleni
  lcd.createChar(1, stupen); //uloz znak stupen do pameti LCD

  Serial.print("EmonHubNode: ");
  Serial.println(myNodeID);
  lcd.setCursor(0, 0);
  lcd.print("  **EMON HUB**  ");
  lcd.setCursor(0, 1);
  lcd.print("ver.4   6.9.2015");

  delay(3000);
  digitalWrite(LEDpin, LOW);
}

//============================Hlavni smycka=====================================
void loop() {


  prijem();
  //zmena obrazovek rucne pomoci tlacitka
  if (analogRead(Tlac) > 500) {
    screen++;
    rucne = true; //po stisku tlacitka rezim zmeny zobrazeni rucne
    rucTime = millis();
    delay(150);
    if (screen >= maxScreen) screen = 1;
  }
  //autom. zmena obrazovek
  currentMillis = millis();
  if ((unsigned long)(currentMillis - lastScreen) >= timeScreen && rucne == false)
  {
    screen++;
    if (screen >= maxScreen) screen = 1;
    lastScreen = currentMillis;
  }
  //ukonceni rucniho rezimu
  if (currentMillis > (rucTime + rucTimeExit)) rucne = false;

  pozadavek ();
  zobrazeni ();
}

//=========================Prijem dat===========================================
//==============================================================================
//zpracovani prijatych dat z jednotlivych nodu
void prijem()
{
  if (ET.receiveData())
  {
    if (emontx.nodeID == 10 && emontx.command == 6) //EmonTx
    {
      data.val.power1 = emontx.data1;
      data.val.power2 = emontx.data2;
      data.val.power3 = emontx.data3;
      data.val.powerAll = emontx.data1 + emontx.data2 + emontx.data3;
      data.val.Vrms =  emontx.data4;
      data.val.tarif = emontx.data5;
      Serial.print("OK");
			Serial.print(" 10");              // toto jsou data z nodu 10
      for (i = 0; i < sizeof(data.val); i++)
      {
        Serial.print(" ");
        Serial.print(data.item[i]);
      }
      Serial.println();
    }

    if (emontx.nodeID == 11 && emontx.command == 6) //Kotel
    {
      data1.val1.TempKotel = emontx.data1;
      data1.val1.TempOut = emontx.data2;
      data1.val1.TempBojler = emontx.data3;
      data1.val1.PodStav = emontx.data4;
      data1.val1.VentStav = emontx.data5;
      data1.val1.TermStav = emontx.data6;
      data1.val1.CerpUTStav = emontx.data7;
      data1.val1.CerpTUVStav = emontx.data8;
      data1.val1.PodM =  emontx.data9;
      data1.val1.VentM = emontx.data10;
      Serial.print("OK");
			Serial.print(" 11");              // toto jsou data z nodu 11
      for (i = 0; i < sizeof(data1.val1); i++)
      {
        Serial.print(" ");
        Serial.print(data1.item1[i]);
      }
      Serial.println();
    }

    if (emontx.nodeID == 12 && emontx.command == 6)//Solar Inverter
    {
      data2.val2.napeti = emontx.data1;
      data2.val2.proud = emontx.data2;
      data2.val2.vykon = emontx.data3;
      data2.val2.vyroba = emontx.data9;
      Serial.print("OK");
			Serial.print(" 12");              // toto jsou data z nodu 12
      for (i = 0; i < sizeof(data2.val2); i++)
      {
        Serial.print(" ");
        Serial.print(data2.item2[i]);
      }
      Serial.println();
    }
    digitalWrite(LEDpin, HIGH); //status LED
    delay(40);
    digitalWrite(LEDpin, LOW);
  }
}

//=====================Odeslani pozadavku na data===============================
//==============================================================================
void pozadavek()
{
  currentMillis = millis();
  if ((unsigned long)(currentMillis - previousMillis) >= period || rucne )
  {

    // zasilani pozadavku na data jednotlivym nodum
    switch (x)
    {
      case 1:
        emontx.nodeID = 15;
        emontx.address = 10;//EmonTx posli data
        emontx.command = 5;
        digitalWrite(TXenableRS485, HIGH);
        ET.sendData();
        digitalWrite(TXenableRS485, LOW);
        break;
      case 2:
        emontx.nodeID = 15;
        emontx.address = 11;//Kotel posli data
        emontx.command = 5;
        digitalWrite(TXenableRS485, HIGH);
        ET.sendData();
        digitalWrite(TXenableRS485, LOW);
        break;
      case 3:
        emontx.nodeID = 15;
        emontx.address = 12;//Sol.Inverter posli data
        emontx.command = 5;
        digitalWrite(TXenableRS485, HIGH);
        ET.sendData();
        digitalWrite(TXenableRS485, LOW);
        break;
    }

    x++;
    if (x >= 4) x = 1;//4
    previousMillis = currentMillis;
  }
}

//===========================Zobrazeni==========================================
//==============================================================================
void zobrazeni()
{

  //obnoveni zobrazovanych udaju
  currentMillis = millis();
  if ((unsigned long)(currentMillis - lastZobraz) >= timeZobraz)
  {
    switch (screen)
    {
      case 1:
        lcd.setCursor(0, 0);
        lcd.print("ELEKTRINA ODBER ");
        lcd.setCursor(0, 1);
        lcd.print("                ");
        lcd.setCursor(0, 1);
        lcd.print(data.val.powerAll);
        lcd.print("W ");
        break;
      case 2:
        lcd.setCursor(0, 0);
        lcd.print("KOTEL  TEPLOTA  ");
        lcd.setCursor(0, 1);
        lcd.print("                ");
        lcd.setCursor(0, 1);
        lcd.print((data1.val1.TempKotel / 10));
        lcd.print(".");
        lcd.print((data1.val1.TempKotel % 10));
        lcd.write(1);//znacka stupne
        lcd.print("C");
        lcd.setCursor(12, 1);
        if (data1.val1.VentStav == 1) {
          lcd.print("TOPI");
        }
        else {
          lcd.print("KLID");
        }
        break;
      case 3:
        lcd.setCursor(0, 0);
        lcd.print("KOTEL  PODAVAC  ");
        lcd.setCursor(0, 1);
        lcd.print("                ");
        lcd.setCursor(0, 1);
        lcd.print(data1.val1.PodM);
        lcd.print("min");
        break;
      case 4:
        lcd.setCursor(0, 0);
        lcd.print("VENKOVNI TEPLOTA");
        lcd.setCursor(0, 1);
        lcd.print("                ");
        lcd.setCursor(0, 1);
        lcd.print((data1.val1.TempOut / 10));
        lcd.print(".");
        lcd.print((abs (data1.val1.TempOut % 10)));
        lcd.write(1);//znacka stupne
        lcd.print("C");
        break;
      case 5:
        lcd.setCursor(0, 0);
        lcd.print("FOTOVOLTAIKA    ");
        lcd.setCursor(0, 1);
        lcd.print("                ");
        lcd.setCursor(0, 1);
        lcd.print(data2.val2.vykon);
        lcd.print("W ");
        break;
      case 6:
        lcd.setCursor(0, 0);
        lcd.print("FOTOVOLTAIKA    ");
        lcd.setCursor(0, 1);
        lcd.print("                ");
        lcd.setCursor(0, 1);
        lcd.print(data2.val2.vyroba / 1000);
        lcd.print(".");
        if (((data2.val2.vyroba % 1000) / 10) < 10) lcd.print("0");
        lcd.print((data2.val2.vyroba % 1000) / 10);
        lcd.print("kWh");
        break;
    }
    lastZobraz = currentMillis;
  }

}




